;===================================
; 6 digits x 7 segments + dot (LED)
; Easy Soft, 01/2002
; jacek.bogusz@ep.com.pl
; Raisonance MA-51 Assembler
;===================================
; Digits are displaying using timer 1 interrupts (about 75Hz). The memory content of buffer
; should be in ascii code. From buffer timer 1 interrupt routine is sending proper digits
; to led display. In my model display contains 6 multiplexed digits. Routine accepts ASCII codes
; from space up to " ^ " character. Use uppers case letters only!!!
; Pins description:
; P1.0 = SERIAL IN (serial input, pin 14)
; P1.1 = SCK (serial clock, pin 11)
; P1.2 = RCK (latch, pin 12)
;
; I've teseted the routine using AT89S8252.

NAME	_6x7driver

$include(REG_8252.pdf)

;stack size
StackAddr		EQU	0E0H

;set timer 1 interrupt frequency to near 75Hz
tmrl1				EQU	0A3H
tmrh1				EQU	0EEH

;pins definitions
dataline			EQU	P1.0
shiftline		EQU	P1.1
latchline		EQU	P1.2

DSEG AT 30H
buffer:			DS		6				;LED's buffer
onLED:			DS		1				;number of digit to display



mainprog SEGMENT CODE INBLOCK

;interrupts
CODE AT 0H
					JMP		InitProc		;reset
CODE AT 1BH
      			JMP		IrqTmr1		;timer 1




CODE AT 30H

;timer 1 routine, which is sending 1 digit in the single call
IrqTmr1:			PUSH	ACC							;save registers
					PUSH	B
					PUSH	PSW
					CLR	TR1							;stop the timer 1
               MOV	TL1,tmrl1					;refresh timer registers contents
               MOV	TH1,tmrh1
               SETB	TR1							;start timer 1 again
               
					CALL	WordSend						;send the data to the display

					INC	onLED							;next digit in the next call
					MOV	A,onLED						;checking if it was 6 digits
					SUBB	A,#6
					JC		IrqTmr1Ret					;if not - exit
					MOV	onLED,#0						;if yes, the next time calling is going to start from first position
					
IrqTmr1Ret:		POP	PSW							;restore registers
					POP	B
					POP	ACC
					RETI


;sending a word ti the display
														;in this moment we are sending the code to switch the proper LED on
WordSend:		ANL	P1,#0F8H					;reset data and clock lines
					MOV	A,onLED					;the number of digit to display is moving to the accumulator
					MOV	DPTR,#digits			;the address of conversion table is moving to DPTR
					MOVC	A,@A+DPTR				;change the accu.content to the proper LED switch-on code

					MOV	B,#8						;B as counter
WSendLoop1:		RLC	A							;CY stores the more sign.bit of the accu.
					MOV	dataline,C				;CY state sets the data line
					SETB	shiftline				;send clock pulse to shift the register content
					CLR	shiftline
					DJNZ	B,WSendLoop1			;was it 8 bits?

					MOV	A,onLED					;in this moment we are sending the character pattern
					MOV	R0,#buffer				;R0 should point to the buffer position; it is depending on
					ADD	A,R0						;digit number to display
					MOV	R0,A
					MOV	A,@R0						;move the character ASCII code to the accu.
					SUBB	A,#' ' 					;my pattern starts from SPACE, so we have to count the offset
					MOV	DPTR,#patterns			;move pattern address to accu.
					MOVC	A,@A+DPTR				;send the second byte, routine is the same as above
					
					MOV	B,#8
WSendLoop2:		RLC	A
					MOV	dataline,C
					SETB	shiftline
					CLR	shiftline
					DJNZ	B,WSendLoop2
					
					SETB	latchline				;move the data from internal register to the 74hct595 outputs
					CLR	latchline

					RET


;main programm
InitProc:		MOV	SP,#StackAddr			;stack size init
					MOV	onLED,#0					;set the digit number to display to 0
					
					MOV	B,#6						;"HELLO" message
					MOV	R0,#buffer
					MOV	DPTR,#ibuf

InitLoop:		CLR	A
					MOVC	A,@A+DPTR
               MOV	@R0,A
               INC	R0
               INC	DPTR
               DJNZ	B,InitLoop
               
               MOV	TMOD,11H					;set timers mode
               MOV	TL1,tmrl1
               MOV	TH1,tmrh1
               SETB	ET1						;enable timer 1 interrupts (disply on)
               SETB	EA							;global interrupt enable
               SETB	TR1						;start timer 1
               
               AJMP	$							;loop and wait for timer 1 interrupts

;patterns
;
;	   d5
;    ======
;d1 |	     | d6
;	 |  d0  |
;	  ======
;d7 |		  | d4
;	 |      |
;	  ======
;	    d2	 O d3
;
;						SP				!				"           #           $           %
patterns:	DB	11111111B,	10110111B,	10111101B,	11101110B,	11000000B,	00111110B	
;						&           '           (				)				*				+
patterns1:	DB	11101111B,	10111111B,	01011001B,	10001011B,	11101111B,	10101110B
;						,				-				.				/				0				1
patterns2:	DB	11110111B,	11111110B,	11110111B,	00111110B,	00001001B,	10101111B
;						2				3				4				5				6				7
patterns3:	DB	00011010B,	10001010B,	10101100B,	11001000B,	01001000B,	10001111B	
;						8				9				:				;				<				=
patterns4:	DB	00001000B,	10001000B,	11111111B,	11111111B,	11111111B,	11111010B
;						>				?				@				A				b				C
patterns5:	DB	11111111B,	00010110B,	00011001B,	00001100B,	01101000B,	01011001B
;						d,				E				F				G				H				I
patterns6:	DB	00101010B,	01011000B,	01011100B,	01001001B,	00101100B,	10101111B
;						J				K				L				M				n				O
patterns7:	DB	10101011B,	01111100B,	01111001B,	01101110B,	01101110B,	00001001B
;						P				Q				r				S				t				U
patterns8:	DB	00011100B,	00000001B,	01111110B,	11001000B,	01111000B,	00101001B
;						V,				W				X				Y				Z				[
pattern9:	DB	00101001B,	00101001B,	00100100B,	10101100B,	00011010B,	01011001B
;						$				]				^
patternsA:	DB	11001000B,	10001011B,	10011100B

;decimal point = d3, digit off = 0xFF

;LEDs switching-on sequence
digits:			DB		0FEH,0FDH,0FBH,0F7H,0EFH,0DFH

;this string is going to be show at the start (i am using this as an example of working)
ibuf:				DB		'H','E','L','L','O','!'

END
