;===================================
; 6 cyfr x 7 segmentw LED
; (C) Easy Soft, 01/2002
;
; Raisonance MA-51 Assembler
;===================================
; Wywietlanie dziaa w oparciu o przerwanie zegara wywoywane z czstotliwoci okoo
; 75Hz. Cyfry s przepisywane z bufora wywietlacza (buffer) w RAM do rejestrw 74HC595
; podczonych w nastpujcy sposb:
; P1.0 = linia danych SERIAL IN (pin 14)
; P1.1 = linia zegara rejestru przesuwnego SCK (pin 11)
; P1.2 = linia zegara rejestru wyjciowego typu latch RCK (pin 12)
;
; Program testowany na AT89S8252.

NAME	_6x7driver

$include(REG_51.pdf)

;stos
StackAddr		EQU	0E0H

;ustawienie czstotliwoci wywoywania przerwania na ok.75Hz
tmrh1				EQU	0E1H

;definicje linii sterujcych
dataline			BIT	P1.0
shiftline		BIT	P1.1
latchline		BIT	P1.2

DSEG AT 30H
buffer:			DS		6				;bufor wywietlacza w RAM
onLED:			DS		1				;numer wywietlanej cyfry



mainprog SEGMENT CODE INBLOCK

;interrupts
CODE AT 0H
					JMP		InitProc		;reset
CODE AT 1BH
      			JMP		IrqTmr1		;timer 1




CODE AT 30H

;procedura obsugi przerwania timera 1, w niej obsuga wywietlacza tzn.wysanie pojedynczej
;cyfry do wywietlenia
IrqTmr1:			PUSH	ACC							;przechowanie zawartoci A oraz sowa kontrolnego programu
					PUSH	B
					PUSH	PSW
               MOV	TH1,tmrh1					;odwieenie zawartoci timer'a 1
               
					ACALL	WordSend						;przesanie do wywietlacza

					INC	onLED							;w nastpnym wywoaniu nastpna cyfra
					MOV	A,onLED						;sprawdzenie,czy byo ju 6 (od 0 do 5) cyfr
					SUBB	A,#6
					JC		IrqTmr1Ret					;jeli nie,to wyjcie
					MOV	onLED,#0						;jeli tak,to nastpnym razem zacznij od pozycji "0"
					
IrqTmr1Ret:		POP	PSW							;odtworzenie zaw.A i PSW
					POP	B
					POP	ACC
					RETI


;wysanie bajtu
WordSend:		ANL	P1,#0F8H					;zerowanie linii danych i zegarowych
					MOV	A,onLED					;do akum.numer wywietlanej cyfry (to jest numer poz.bufora)
					MOV	DPTR,#digits			;zamiana numeru cyfry na odpowiedni kod numeru wywietlacza
					MOVC	A,@A+DPTR

					MOV	B,#8						;rejestr B jako licznik operacji
WSendLoop1:		RLC	A							;do CY stan bitu
					MOV	dataline,C				;CY ustawia stan linii danych
					SETB	shiftline
					CLR	shiftline
					DJNZ	B,WSendLoop1

					MOV	A,onLED					;do akum.numer pozycji w buforze
					MOV	R0,#buffer				;wyliczenie offsetu,R0 jako wskanik pozycji w buforze
					ADD	A,R0
					MOV	R0,A
					MOV	A,@R0						;przepisanie kodu znaku z bufora do A
					SUBB	A,#' ' 					;od akumulatora odejmujemy kod " " aby wyliczy offset generatora znakw
					MOV	DPTR,#patterns			;do DPTR adres pocztku kodw generatora
					MOVC	A,@A+DPTR				;wysanie 2-go bajtu
					
					MOV	B,#8						;rejestr B jako licznik operacji
WSendLoop2:		RLC	A							;do CY stan bit
					MOV	dataline,C				;CY ustawia stan linii danych
					SETB	shiftline
					CLR	shiftline
					DJNZ	B,WSendLoop2
					
					SETB	latchline				;zapis do wyj rejestrw
					CLR	latchline

					RET


;pocztek programu gwnego
InitProc:		MOV	SP,#StackAddr			;inicjowanie rozmiaru stosu
					MOV	onLED,#0					;do wywietlenia cyfra numer "0"
					
					MOV	B,#6						;6 bajtw do przepisania
					MOV	R0,#buffer
					MOV	DPTR,#ibuf

InitLoop:		CLR	A
					MOVC	A,@A+DPTR
               MOV	@R0,A
               INC	R0
               INC	DPTR
               DJNZ	B,InitLoop
               
               MOV	TMOD,11H					;oba timery jako 16 bitowe,kontrolowane wewntrznie
               MOV	TH1,tmrh1				;przerwanie wywoywane z czstotliw.okoo 75Hz
               SETB	ET1						;zezwolenie na przerwanie od timer'a 1
               SETB	EA							;zezwolenie na przyjmowanie przerwa
               SETB	TR1						;uruchomienie timer'a 1
               
               AJMP	$

;tutaj wzorce cyfr
;
;	   d5
;    ======
;d1 |	     | d6
;	 |  d0  |
;	  ======
;d7 |		  | d4
;	 |      |
;	  ======
;	    d2	 O d3
;
;						SP				!				"           #           $           %
patterns:	DB	11111111B,	10110111B,	10111101B,	11101110B,	11000000B,	00111110B	
;						&           '           (				)				*				+
patterns1:	DB	11101111B,	10111111B,	01011001B,	10001011B,	11101111B,	10101110B
;						,				-				.				/				0				1
patterns2:	DB	11110111B,	11111110B,	11110111B,	00111110B,	00001001B,	10101111B
;						2				3				4				5				6				7
patterns3:	DB	00011010B,	10001010B,	10101100B,	11001000B,	01001000B,	10001111B	
;						8				9				:				;				<				=
patterns4:	DB	00001000B,	10001000B,	11111111B,	11111111B,	11111111B,	11111010B
;						>				?				@				A				b				C
patterns5:	DB	11111111B,	00010110B,	00011001B,	00001100B,	01101000B,	01011001B
;						d,				E				F				G				H				I
patterns6:	DB	00101010B,	01011000B,	01011100B,	01001001B,	00101100B,	10101111B
;						J				K				L				M				n				O
patterns7:	DB	10101011B,	01111100B,	01111001B,	01101110B,	01101110B,	00001001B
;						P				Q				r				S				t				U
patterns8:	DB	00011100B,	00000001B,	01111110B,	11001000B,	01111000B,	00101001B
;						V,				W				X				Y				Z				[
pattern9:	DB	00101001B,	00101001B,	00100100B,	10101100B,	00011010B,	01011001B
;						$				]				^
patternsA:	DB	11001000B,	10001011B,	10011100B

;przecinek = d3, wyczenie cyfry = 0xFF

;kolejno zaczania
digits:			DB		0FEH,0FDH,0FBH,0F7H,0EFH,0DFH

;zawarto wywietlacza po wczeniu (przepisywana do bufora po reset)
ibuf:				DB		'H','E','L','L','O','!'

END
